import React, { useRef, useCallback, useEffect, useState } from 'react';
import * as ReactDOM from 'react-dom';
import { PortalManager } from '@highlight-ui/utils-portal-manager';
import classnames from 'classnames';
import { IconButton } from '@highlight-ui/button';
import { Icon } from '@highlight-ui/icon';
import { Typography } from '@highlight-ui/typography';
import { parseMetadata, suffixMetadata, noop } from '@highlight-ui/utils-commons';

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function () {
  __assign = Object.assign || function __assign(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];
      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }
    return t;
  };
  return __assign.apply(this, arguments);
};
function __spreadArray(to, from, pack) {
  if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
    if (ar || !(i in from)) {
      if (!ar) ar = Array.prototype.slice.call(from, 0, i);
      ar[i] = from[i];
    }
  }
  return to.concat(ar || Array.prototype.slice.call(from));
}
var variantToIconNameMap = {
  highlight: 'info-circle',
  success: 'check-circle',
  warning: 'exclamation-triangle',
  critical: 'exclamation-circle'
};
var variantToIconName = function (variant) {
  return variantToIconNameMap[variant];
};
var themeToVariant = function (theme) {
  switch (theme) {
    case 'danger':
      return 'critical';
    case 'success':
      return 'success';
    case 'warning':
      return 'warning';
    case 'info':
    default:
      return 'highlight';
  }
};
function createTimer(callback, delay) {
  var timerId = 0;
  var start = new Date(0);
  var remaining = delay;
  var timer = {
    reset: function () {
      clearTimeout(timerId);
      timerId = 0;
    },
    pause: function () {
      window.clearTimeout(timerId);
      remaining -= +new Date() - +start;
    },
    resume: function () {
      start = new Date();
      window.clearTimeout(timerId);
      timerId = window.setTimeout(callback, remaining);
    }
  };
  timer.resume();
  return timer;
}
var styles$1 = {
  "toast": "Toast-module__7Rsmf0Z-__v5-3-15",
  "open-animation": "Toast-module__QAOqyVC8__v5-3-15",
  "close-animation": "Toast-module__Fb2YHZ9q__v5-3-15",
  "progressBarInner": "Toast-module__u5AhouW6__v5-3-15",
  "container": "Toast-module__0SfwgGXF__v5-3-15",
  "statusIcon": "Toast-module__wZ-d-u5S__v5-3-15",
  "contentText": "Toast-module__dK0lhJcy__v5-3-15",
  "closeButton": "Toast-module__biMx2up5__v5-3-15",
  "progressBarContainer": "Toast-module__orMLA3vn__v5-3-15",
  "progress-bar": "Toast-module__G8ruDrV2__v5-3-15",
  "highlight": "Toast-module__xSpF72H1__v5-3-15",
  "success": "Toast-module__fZ89ogVl__v5-3-15",
  "warning": "Toast-module__jCEGPIn-__v5-3-15",
  "critical": "Toast-module__FWC-mr-v__v5-3-15",
  "actionsContainer": "Toast-module__NoH-Wske__v5-3-15"
};
var LEAVING_ANIMATION_DURATION = 200;
var Toast = function (_a) {
  var _b = _a.variant,
    variant = _b === void 0 ? 'highlight' : _b,
    _c = _a.autoClose,
    autoClose = _c === void 0 ? true : _c,
    _d = _a.duration,
    duration = _d === void 0 ? 5000 : _d,
    _e = _a.showCloseButton,
    showCloseButton = _e === void 0 ? true : _e,
    actions = _a.actions,
    _f = _a.onOpen,
    onOpen = _f === void 0 ? noop : _f,
    _g = _a.onClose,
    onClose = _g === void 0 ? noop : _g,
    className = _a.className,
    children = _a.children,
    metadata = _a.metadata,
    _h = _a.onRequestToClose,
    onRequestToClose = _h === void 0 ? noop : _h,
    _j = _a.onRequestToRemove,
    onRequestToRemove = _j === void 0 ? noop : _j,
    theme = _a.theme,
    _k = _a.isVisible,
    isVisible = _k === void 0 ? true : _k;
  var isVisibleRef = useRef(isVisible);
  var closeTimerRef = useRef(null);
  var removeTimerRef = useRef(null);
  var clearCloseTimer = useCallback(function () {
    if (removeTimerRef.current) {
      clearTimeout(removeTimerRef.current);
    }
    if (closeTimerRef.current) {
      closeTimerRef.current.reset();
    }
  }, []);
  var close = useCallback(function () {
    clearCloseTimer();
    onRequestToClose();
  }, [clearCloseTimer, onRequestToClose]);
  var startCloseTimer = useCallback(function () {
    if (autoClose && duration) {
      closeTimerRef.current = createTimer(close, duration);
    }
  }, [autoClose, close, duration]);
  useEffect(function () {
    onOpen();
    startCloseTimer();
    return function () {
      onClose();
      clearCloseTimer();
    };
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);
  useEffect(function () {
    if (isVisibleRef.current !== isVisible) {
      isVisibleRef.current = isVisible;
      if (!isVisible) {
        removeTimerRef.current = setTimeout(onRequestToRemove, LEAVING_ANIMATION_DURATION);
      }
    }
  }, [isVisible, onRequestToRemove]);
  var handleMouseEnter = function () {
    if (closeTimerRef.current) {
      closeTimerRef.current.pause();
    }
  };
  var handleMouseLeave = function () {
    if (closeTimerRef.current) {
      closeTimerRef.current.resume();
    }
  };
  var variantOrDeprecatedTheme = typeof theme !== 'undefined' ? themeToVariant(theme) : variant;
  return /*#__PURE__*/React.createElement("div", __assign({
    "data-state": isVisible ? 'entering' : 'leaving',
    role: "log",
    className: classnames(className, styles$1.toast),
    onMouseEnter: handleMouseEnter,
    onMouseLeave: handleMouseLeave
  }, parseMetadata(metadata)), /*#__PURE__*/React.createElement("div", {
    className: styles$1.container
  }, /*#__PURE__*/React.createElement(Icon, {
    className: styles$1.statusIcon,
    name: variantToIconName(variantOrDeprecatedTheme),
    color: "icon-" + variantOrDeprecatedTheme,
    size: 20,
    type: "solid",
    metadata: suffixMetadata(metadata, 'icon')
  }), children && /*#__PURE__*/React.createElement(Typography, {
    token: "body-base",
    color: "text-default",
    className: styles$1.contentText
  }, children), showCloseButton && /*#__PURE__*/React.createElement(IconButton, __assign({
    "aria-label": "close",
    icon: "times",
    iconColor: "icon-on-secondary",
    type: "button",
    className: styles$1.closeButton,
    tabIndex: -1,
    onClick: onRequestToClose
  }, parseMetadata(suffixMetadata(metadata, 'close-btn'))))), actions && /*#__PURE__*/React.createElement("div", __assign({
    className: styles$1.actionsContainer
  }, parseMetadata(suffixMetadata(metadata, 'actions-container'))), actions), autoClose && duration && /*#__PURE__*/React.createElement("div", __assign({
    className: styles$1.progressBarContainer,
    role: "progressbar"
  }, parseMetadata(suffixMetadata(metadata, 'progress-bar'))), /*#__PURE__*/React.createElement("div", {
    className: classnames(styles$1.progressBarInner, styles$1[variantOrDeprecatedTheme]),
    style: {
      animationDuration: duration / 1000 + "s"
    }
  })));
};
var styles = {
  "container": "ToastManager-module__6kXasj8k__v5-3-15"
};
var hasCustomId = function (settings) {
  return Object.keys(settings).includes('id');
};
var ToastManager = function (_a) {
  var bindNotify = _a.bindNotify,
    bindGetToasts = _a.bindGetToasts,
    bindCloseAll = _a.bindCloseAll;
  var _b = useState([]),
    toasts = _b[0],
    setToasts = _b[1];
  var idCounterRef = useRef(0);
  var closeToast = function (id) {
    setToasts(function (prevState) {
      return prevState.map(function (toast) {
        if (toast.id === id) return __assign(__assign({}, toast), {
          isVisible: false
        });
        return toast;
      });
    });
  };
  var removeToast = function (id) {
    setToasts(function (prevState) {
      return prevState.filter(function (toast) {
        return toast.id !== id;
      });
    });
  };
  var createToastInstance = function (options) {
    if (options === void 0) {
      options = {
        children: null
      };
    }
    idCounterRef.current += 1;
    var uniqueId = idCounterRef.current;
    var id = hasCustomId(options) ? (options.id || '') + "-" + uniqueId : String(uniqueId);
    return __assign(__assign({}, options), {
      id: id,
      onRequestToClose: function () {
        return closeToast(id);
      },
      onRequestToRemove: function () {
        return removeToast(id);
      }
    });
  };
  var notify = function (options) {
    // If there's a custom toast ID passed, close existing toasts with the same custom ID
    if (hasCustomId(options)) {
      toasts.forEach(function (toast) {
        // Since unique ID is still appended to a custom ID, skip the unique ID and check only prefix
        if (toast.id && String(toast.id).startsWith(options.id || '')) {
          closeToast(toast.id);
        }
      });
    }
    var toast = createToastInstance(options);
    setToasts(function (prevState) {
      return __spreadArray([toast], prevState, true);
    });
    return toast;
  };
  var closeAll = function () {
    toasts.forEach(function (_a) {
      var onRequestToClose = _a.onRequestToClose;
      return onRequestToClose && onRequestToClose();
    });
  };
  var getToasts = function () {
    return toasts;
  };
  useEffect(function () {
    bindNotify(notify);
    bindGetToasts(getToasts);
    bindCloseAll(closeAll);
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [toasts]);
  return /*#__PURE__*/React.createElement("div", {
    className: styles.container
  }, toasts.map(function (props) {
    return /*#__PURE__*/React.createElement(Toast, __assign({
      key: props.id
    }, props));
  }));
};
var Toaster = /** @class */function () {
  function Toaster() {
    var _this = this;
    this.closeAllHandler = function () {};
    this.getToastsHandler = function () {
      return [];
    };
    this.notifyHandler = function () {};
    this.bindNotify = function (handler) {
      _this.notifyHandler = handler;
    };
    this.bindGetToasts = function (handler) {
      _this.getToastsHandler = handler;
    };
    this.bindCloseAll = function (handler) {
      _this.closeAllHandler = handler;
    };
    this.closeAll = function () {
      _this.closeAllHandler();
    };
    this.notify = function (options) {
      _this.notifyHandler(options);
    };
    var container = document.createElement('div');
    container.setAttribute('personio-toaster-container', '');
    PortalManager.containerInstance().appendChild(container);
    ReactDOM.render( /*#__PURE__*/React.createElement(ToastManager, {
      bindNotify: this.bindNotify,
      bindGetToasts: this.bindGetToasts,
      bindCloseAll: this.bindCloseAll
    }), container);
    return this;
  }
  Object.defineProperty(Toaster.prototype, "toasts", {
    get: function () {
      return this.getToastsHandler();
    },
    enumerable: false,
    configurable: true
  });
  return Toaster;
}();
var Toaster$1 = new Toaster();
export { Toaster$1 as toaster };
//# sourceMappingURL=index.js.map
